@@ -30,6 +30,12 @@ function install_fwaas() {
     # Install the service.
     :
     setup_develop $DEST/neutron-fwaas
+    if is_ubuntu; then
+        install_package libnetfilter-log1
+    else
+        # EPEL
+        install_package libnetfilter_log
+    fi
 }
 
 function configure_fwaas_v1() {
@@ -43,11 +49,20 @@ function configure_fwaas_v1() {
 function configure_fwaas_v2() {
     # Add conf file
     cp $NEUTRON_FWAAS_DIR/etc/neutron_fwaas.conf.sample $NEUTRON_FWAAS_CONF
+    neutron_server_config_add $NEUTRON_FWAAS_CONF
+    inicomment $NEUTRON_FWAAS_CONF service_providers service_provider
+    iniadd $NEUTRON_FWAAS_CONF service_providers service_provider $NEUTRON_FWAAS_SERVICE_PROVIDERV2
+
     neutron_fwaas_configure_driver fwaas_v2
     iniset_multiline $Q_L3_CONF_FILE fwaas agent_version v2
     iniset_multiline $Q_L3_CONF_FILE fwaas driver $FWAAS_DRIVER_V2
-    iniset $NEUTRON_CORE_PLUGIN_CONF fwaas firewall_l2_driver $FW_L2_DRIVER
-    iniset $NEUTRON_CORE_PLUGIN_CONF agent extensions fwaas_v2
+    #TODO(hoangcx) we can remove the slashes below once neutron-legacy has gone
+    iniset /$NEUTRON_CORE_PLUGIN_CONF fwaas firewall_l2_driver $FW_L2_DRIVER
+    iniset /$NEUTRON_CORE_PLUGIN_CONF agent extensions fwaas_v2
+}
+
+function configure_l3_log_fwaas_v2(){
+    iniadd $Q_L3_CONF_FILE agent extensions fwaas_v2_log
 }
 
 function neutron_fwaas_generate_config_files {
@@ -73,6 +88,12 @@ function shutdown_fwaas() {
 function cleanup_fwaas() {
     # Cleanup the service.
     :
+    if is_ubuntu; then
+        uninstall_package libnetfilter-log1
+    else
+        # EPEL
+        uninstall_package libnetfilter_log
+    fi
 }
 
 function neutron_fwaas_configure_common {
@@ -109,6 +130,10 @@ if is_service_enabled q-svc neutron-api && is_service_enabled q-fwaas q-fwaas-v1
         elif is_service_enabled q-fwaas-v2 neutron-fwaas-v2; then
             echo_summary "Configuring neutron-fwaas for FWaaS v2"
             configure_fwaas_v2
+            if is_service_enabled q-log neutron-log; then
+                echo_summary "Configuring FwaaS V2 packet log for l3 extension"
+                configure_l3_log_fwaas_v2
+            fi
         else
             echo_summary "Configuring neutron-fwaas for FWaaS v1"
             configure_fwaas_v1
